//
// Created by zx on 2020/7/7.
//

#include <Parkspace_communicator.h>
#include "StoreProcessTask.h"
#include "process_message.pb.h"
#include "command_manager.h"

StoreProcessTask::StoreProcessTask()
{
    m_command_id=-1;
    m_terminor_id=-1;
    mb_completed=false;
}

StoreProcessTask::~StoreProcessTask()
{

}

Error_manager StoreProcessTask::init_task(unsigned int command_id, unsigned int terminor_id,
        message::Locate_information locate_info,message::Car_info car_info)
{
    m_command_id=command_id;
    m_terminor_id=terminor_id;
    m_car_info=car_info;
    m_locate_info=locate_info;
    return SUCCESS;
}

Error_manager StoreProcessTask::locate_step() {
    message::Measure_request_msg request;
    message::Base_info base_info;
    base_info.set_msg_type(message::eLocate_request_msg);
    base_info.set_sender(message::eMain);
    base_info.set_receiver(message::eMeasurer);
    base_info.set_timeout_ms(5000); //测量超时5s
    request.mutable_base_info()->CopyFrom(base_info);

    request.set_command_id(m_command_id);
    request.set_terminal_id(m_terminor_id);

    Error_manager code=Locate_communicator::get_instance_pointer()->locate_request(request,m_measure_response_msg);
    if(code!=SUCCESS)
        return code;

    if(m_measure_response_msg.error_manager().error_code()==0) {
        /*LOG(INFO)<<"测量成功,停车终端:"<<m_terminor_id
                 <<", 指令id:"<<m_command_id
                 <<", 车辆宽:"<<m_measure_response_msg.locate_information().locate_width()
                 <<", 车辆高:"<<m_measure_response_msg.locate_information().locate_height()
                 <<", 车辆角度:"<<m_measure_response_msg.locate_information().locate_angle()
                 <<", 车牌号:"<<m_parcspace_alloc_response_msg.allocated_space_info().car_info().license();*/
        return SUCCESS;
    }
    else
        return Error_manager(FAILED,MINOR_ERROR,"measure response error_code error");

}

/*
     * 分配车位
     */
Error_manager StoreProcessTask::alloc_space()
{
    /*
     * 检查是否有测量数据
     */
    if(m_locate_info.has_locate_height()==false||m_locate_info.has_locate_width()==false)
    {
        return Error_manager(FAILED,MINOR_ERROR," parkspace alloc request without measure info");
    }

    message::Parkspace_allocation_request_msg request;
    message::Base_info base_info;
    base_info.set_msg_type(message::eParkspace_allocation_request_msg);
    base_info.set_sender(message::eMain);
    base_info.set_receiver(message::eParkspace);
    base_info.set_timeout_ms(1000); //测量超时1s
    request.mutable_base_info()->CopyFrom(base_info);

    request.mutable_car_info()->CopyFrom(m_car_info);

    request.set_command_id(m_command_id);
    request.set_terminal_id(m_terminor_id);

    Error_manager code=Parkspace_communicator::get_instance_pointer()->alloc_request(request,m_parcspace_alloc_response_msg);
    if(code!=SUCCESS)
        return code;

    if(m_parcspace_alloc_response_msg.error_manager().error_code()==0)
    {
        /*LOG(INFO)<<"分配车位成功,停车终端:"<<m_terminor_id
            <<", 指令id:"<<m_command_id
            <<", 车位楼层:"<<m_parcspace_alloc_response_msg.allocated_space_info().floor()
            <<", 车位序号:"<<m_parcspace_alloc_response_msg.allocated_space_info().index()
            <<", 车牌号:"<<m_parcspace_alloc_response_msg.allocated_space_info().car_info().license();*/
        return SUCCESS;
    }
    else
        return Error_manager(FAILED,MINOR_ERROR,"parkspace alloc response error_code error");
}

/*
 * 车位解锁,当停车失败时需要车位解锁
 */
Error_manager StoreProcessTask::release_space_step()
{
    /*
    * 检查是否曾经分配过车位
    */
    if(m_parcspace_alloc_response_msg.has_allocated_space_info()==false)
    {
        return Error_manager(FAILED,MINOR_ERROR," parkspace release request without space info");
    }

    message::Parkspace_release_request_msg request;
    message::Base_info base_info;
    base_info.set_msg_type(message::eParkspace_release_request_msg);
    base_info.set_sender(message::eMain);
    base_info.set_receiver(message::eParkspace);
    base_info.set_timeout_ms(1000); //测量超时1s
    request.mutable_base_info()->CopyFrom(base_info);

    message::Parkspace_info space_info=m_parcspace_alloc_response_msg.allocated_space_info();
    request.mutable_release_space_info()->CopyFrom(space_info);

    request.set_command_id(m_command_id);


    message::Parkspace_release_response_msg release_response;
    Error_manager code=Parkspace_communicator::get_instance_pointer()->release_request(request,release_response);
    if(code!=SUCCESS)
        return code;

    if(release_response.error_manager().error_code()==0) {
        LOG(ERROR)<<"停车流程异常,释放车位成功,停车终端:"<<m_terminor_id
                 <<", 指令id:"<<m_command_id
                 <<", 车位楼层:"<<m_parcspace_alloc_response_msg.allocated_space_info().floor()
                 <<", 车位序号:"<<m_parcspace_alloc_response_msg.allocated_space_info().index()
                 <<", 车牌号:"<<m_parcspace_alloc_response_msg.allocated_space_info().car_info().license();
        return SUCCESS;
    }
    else
        return Error_manager(FAILED,MINOR_ERROR,"parkspace release response error_code error");
}

void StoreProcessTask::Main()
{
    /*
     * 外部已经分配好车位,进入到此流程说明车位已经分配好, 存放在 m_parcspace_alloc_response_msg
     */
    Error_manager code;
    Command_manager::get_instance_pointer()->updata_store_entrance_statu(m_terminor_id,message::eWorking);
    //开始执行停车指令
    //第一步测量
    switch (0)
    {
        //第一步,测量
        case 0:
        {
            code=locate_step();
            if(code!=SUCCESS)
            {
                LOG(ERROR)<<"测量失败:"<<code.to_string();
                break;
            }
        }
        //第二步,调度
        case 1:
        {
            //暂停 5-15s 代替
            int delay=rand()%5000+5000;
            usleep(1000*delay);
            //调度失败,break,完成return
        }
        //第三步,占据车位
        case 2:
        {
            ////
            Command_manager::get_instance_pointer()->updata_store_entrance_statu(m_terminor_id,message::eReady);
            LOG(INFO)<<"停车成功,停车终端:"<<m_terminor_id<<"指令id:"<<m_command_id
                     <<", 车位id:"<<m_parcspace_alloc_response_msg.allocated_space_info().parkspace_id()
                     <<", 车位楼层:"<<m_parcspace_alloc_response_msg.allocated_space_info().floor()
                     <<", 车位序号:"<<m_parcspace_alloc_response_msg.allocated_space_info().index()
                     <<", 车牌号:"<<m_car_info.license()
                     <<", 库内车牌号:"<<m_parcspace_alloc_response_msg.allocated_space_info().car_info().license();
            return ;
        }
    }

    /*
     * switch 语句break,说明停车故障,需清除车位
     */

    //失败,清理车位
    code=release_space_step();
    if(code!=SUCCESS)
    {
        LOG(ERROR)<<"致命故障,停车失败,清理车位故障:"<<code.to_string();
        /*
         * 此处应暂停系统,待管理员介入
         */
    }


}
